/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial_user.c
* Version      : CodeGenerator for RL78/G13 V2.05.03.01 [12 Nov 2018]
* Device(s)    : R5F100LL
* Tool-Chain   : CCRL
* Description  : This file implements device driver for Serial module.
* Creation Date: 2019/11/13
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
#pragma interrupt r_csi11_interrupt(vect=INTCSI11)
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint8_t * gp_csi11_rx_address;         /* csi11 receive buffer address */
extern volatile uint16_t  g_csi11_rx_length;           /* csi11 receive data length */
extern volatile uint16_t  g_csi11_rx_count;            /* csi11 receive data count */
extern volatile uint8_t * gp_csi11_tx_address;         /* csi11 send buffer address */
extern volatile uint16_t  g_csi11_send_length;         /* csi11 send data length */
extern volatile uint16_t  g_csi11_tx_count;            /* csi11 send data count */
/* Start user code for global. Do not edit comment generated here */
extern volatile uint8_t  g_csi11_comflag;              /* csi00 transfer mode flag */
extern uint8_t tx_bufA[4];
extern uint8_t rx_bufA[4];
uint8_t dummy_add;
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: r_csi11_interrupt
* Description  : This function is INTCSI11 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void __near r_csi11_interrupt(void)
{
    volatile uint8_t err_type;
    volatile uint8_t sio_dummy;

    err_type = (uint8_t)(SSR03 & _0001_SAU_OVERRUN_ERROR);
    SIR03 = (uint16_t)err_type;
    dummy_add = tx_bufA[1] & 0xc0;
    if (1U == err_type)
    {
        r_csi11_callback_error(err_type);    /* overrun error occurs */
    }
    else
    {
        if (g_csi11_tx_count == 3U)
        {
	    ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
	    SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _3000_SAU_TIMING_4 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	    SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
            if (0U != gp_csi11_rx_address)
            {
                *gp_csi11_rx_address = SIO11;
                gp_csi11_rx_address++;
            }
            else
            {
                sio_dummy = SIO11;
            }

            if (0U != gp_csi11_tx_address)
            {
                SIO11 = *gp_csi11_tx_address;
                gp_csi11_tx_address++;
            }
            else
            {
                SIO11 = 0xFFU;
            }

            g_csi11_tx_count--;
        }else if (g_csi11_tx_count == 2U)
        {
	    
	    if(dummy_add == 0x80)
	    {
		ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
		SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _1000_SAU_TIMING_2 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	        SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
	    	if (0U != gp_csi11_rx_address)
            	{
                	*gp_csi11_rx_address = SIO11;
                	gp_csi11_rx_address++;
            	}
            	else
            	{
                	sio_dummy = SIO11;
            	}

            	if (0U != gp_csi11_tx_address)
            	{
                	SIO11 = *gp_csi11_tx_address;
                	gp_csi11_tx_address++;
            	}
            	else
            	{
                	SIO11 = 0xFFU;
            	}
	    }
	    else
	    {
		ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
		SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _3000_SAU_TIMING_4 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	        SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
		if (0U != gp_csi11_rx_address)
            	{
                	*gp_csi11_rx_address = SIO11;
                	gp_csi11_rx_address++;
            	}
            	else
            	{
                	sio_dummy = SIO11;
            	}

            	if (0U != gp_csi11_tx_address)
            	{
                	SIO11 = *gp_csi11_tx_address;
                	gp_csi11_tx_address++;
            	}
            	else
            	{
                	SIO11 = 0xFFU;
            	}
	    }
            g_csi11_tx_count--;
        }else if (g_csi11_tx_count == 1U)
        {
            if(dummy_add == 0x80)
	    {
		ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
		SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _1000_SAU_TIMING_2 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	        SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
	    	if (0U != gp_csi11_rx_address)
            	{
                	*gp_csi11_rx_address = SIO11;
                	gp_csi11_rx_address++;
            	}
            	else
            	{
                	sio_dummy = SIO11;
            	}

            	if (0U != gp_csi11_tx_address)
            	{
                	SIO11 = *gp_csi11_tx_address;
                	gp_csi11_tx_address++;
            	}
            	else
            	{
                	SIO11 = 0xFFU;
            	}
	    }
	    else
	    {
		ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
		SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _3000_SAU_TIMING_4 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	        SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
		if (0U != gp_csi11_rx_address)
            	{
                	*gp_csi11_rx_address = SIO11;
                	gp_csi11_rx_address++;
            	}
            	else
            	{
                	sio_dummy = SIO11;
            	}

            	if (0U != gp_csi11_tx_address)
            	{
                	SIO11 = *gp_csi11_tx_address;
                	gp_csi11_tx_address++;
            	}
            	else
            	{
                	SIO11 = 0xFFU;
            	}
	    }
            g_csi11_tx_count--;
        }
        else 
        {
            if (0U == g_csi11_tx_count)
            {
		ST0 |= _0008_SAU_CH3_STOP_TRG_ON;        	/* disable CSI11 */
		SCR03 = _C000_SAU_RECEPTION_TRANSMISSION | _3000_SAU_TIMING_4 | _0000_SAU_MSB | _0007_SAU_LENGTH_8;
	        SS0 |= _0008_SAU_CH3_START_TRG_ON;             	/* enable CSI11 */
                if (0U != gp_csi11_rx_address)
                {
                    *gp_csi11_rx_address = SIO11;
		    g_csi11_comflag = 0x00;             /* clear transfer flag */
                }
                else
                {
                    sio_dummy = SIO11;
                }
            }

            r_csi11_callback_sendend();    /* complete send */
            r_csi11_callback_receiveend();    /* complete receive */
        }
    }
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_receiveend
* Description  : This function is a callback function when CSI11 finishes reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_receiveend(void)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_error
* Description  : This function is a callback function when CSI11 reception error occurs.
* Arguments    : err_type -
*                    error type value
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_error(uint8_t err_type)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_sendend
* Description  : This function is a callback function when CSI11 finishes transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_sendend(void)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
